from pylab import *
from bisect import *
import dolfin

def piecewise_linear_adapter(tarray, uarray):
    """Returns the piecewise linear interpolant described by tarray
    and uarray as a function which can be evaluated at an arbitrary point."""
    def u_adapted(t):
        # Find the endpoints of the containing interval
        i1 = bisect_right(tarray, t)
        if(i1 == size(tarray)):
            i1 -= 1
        i0 = i1 - 1

        t0 = tarray[i0]
        t1 = tarray[i1]

        u0 = uarray[i0]
        u1 = uarray[i1]

        # Lagrange polynomial weights
        w0 = (t1 - t) / (t1 - t0)
        w1 = (t - t0) / (t1 - t0)

        ut = w0*u0 + w1*u1
        return ut

    # Return vectorized (accepting array argument) function
    return vectorize(u_adapted)

def piecewise_constant_adapter(tarray, uarray):
    """Returns the piecewise constant interpolant described by tarray
    and uarray as a function which can be evaluated at an arbitrary point."""
    def u_adapted(t):
        # Find the endpoints of the containing interval
        i1 = bisect_right(tarray, t)
        if(i1 == size(tarray)):
            i1 -= 1
        i0 = i1 - 1

        t0 = tarray[i0]
        t1 = tarray[i1]

        u0 = uarray[i0]
        u1 = uarray[i1]

        ut = u1
        return ut

    # Return vectorized (accepting array argument) function
    return vectorize(u_adapted)

def f3(t):
    return pow(t, 3)

tarray = linspace(0.0, 4.0, 5)
uarray = f3(tarray)

f3_p0 = piecewise_constant_adapter(tarray, uarray)
f3_p1 = piecewise_linear_adapter(tarray, uarray)

tarray_fine = linspace(0.0, 4.0, 1000)
uarray_fine = f3(tarray_fine)
uarray_fine_p0 = f3_p0(tarray_fine)
uarray_fine_p1 = f3_p1(tarray_fine)

subplot(311)
plot(tarray_fine, uarray_fine,
     tarray_fine, uarray_fine_p0,
     tarray_fine, uarray_fine_p1)
title("piecewise polynomial interpolation (constant and linear)")

# Construct a piecewise linear interpolant as a linear combination of
# hat basis functions.  Note that this may not the most efficient way
# of representing piecewise functions, but for understanding the
# concept it is a good representation.
uarray0 = zeros(size(tarray))
uarray0[0] = 1.0
basis0 = piecewise_linear_adapter(tarray, uarray0)
uarray1 = zeros(size(tarray))
uarray1[1] = 1.0
basis1 = piecewise_linear_adapter(tarray, uarray1)
uarray2 = zeros(size(tarray))
uarray2[2] = 1.0
basis2 = piecewise_linear_adapter(tarray, uarray2)
uarray3 = zeros(size(tarray))
uarray3[3] = 1.0
basis3 = piecewise_linear_adapter(tarray, uarray3)
uarray4 = zeros(size(tarray))
uarray4[4] = 1.0
basis4 = piecewise_linear_adapter(tarray, uarray4)

subplot(312)
plot(tarray, uarray1, '--',
     tarray, uarray2, '-',
     tarray, uarray3, '--')
title("hat basis functions")
xlim(-0.5, 4.5)
ylim(-0.5, 1.5)

# Define a piecewise linear function as a linear combination of
# hat basis functions
def linear_combination(t):
    return basis0(t) * 0.0 + basis1(t) * 2.0 + basis2(t) * 1.0 + \
        basis3(t) * -1.0 + basis4(t) * 4.0

tarray_fine = linspace(0.0, 4.0, 1000)
uarray_fine = linear_combination(tarray_fine)

subplot(313)
plot(tarray_fine, uarray_fine)
xlim(-0.5, 4.5)
ylim(-1.5, 4.5)
title(("linear combination of hat functions"))
show()

mesh = dolfin.UnitSquare(4, 4)

V = dolfin.FunctionSpace(mesh, "CG", 1)
U = dolfin.Function(V)
xi = U.vector()

xi.array()

def f_2D(x, y):
    return x*y

# Demo of hat function in 2D
for v in dolfin.vertices(mesh):
    p = v.point()
    i = v.index()
    if(i == 18):
        val = 1.0
    else:
        val = 0.0
    xi[i] = val

file = dolfin.File("output.pvd")
file << U

# Demo of looping over triangles (cells) and their vertices
integral = 0.0
for c in dolfin.cells(mesh):
    cell_integral = 0.0
    for v in dolfin.vertices(c):
        p = v.point()
        print "vertex %d of cell %d has coordinates (%f, %f)" \
            % (v.index(), c.index(), p[0], p[1])
        cell_integral += 0.0
    integral += 0.0

dolfin.plot(U, wireframe=True, interactive=True)
